package jalview.ws.slivkaws;

import jalview.datamodel.AlignmentView;
import jalview.gui.AlignFrame;
import jalview.ws.WSMenuEntryProviderI;
import jalview.ws.jws2.MsaWSClient;

import java.awt.event.ActionEvent;
import java.io.IOError;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import uk.ac.dundee.compbio.slivkaclient.SlivkaClient;

public class SlivkaWSDiscoverer
  implements Runnable, WSMenuEntryProviderI
{
  private static SlivkaWSDiscoverer instance = null;
  private SlivkaClient client;

  private SlivkaWSDiscoverer() {
    try
    {
      client = new SlivkaClient("gjb-www-1.cluster.lifesci.dundee.ac.uk", 3203);
    } catch (URISyntaxException e)
    {
      throw new RuntimeException(e);
    }
  }

  public static SlivkaWSDiscoverer getInstance()
  {
    if (instance == null) {
		instance = new SlivkaWSDiscoverer();
	}
    return instance;
  }

  @Override
  public void attachWSMenuEntry(JMenu wsmenu, final AlignFrame alignFrame)
  {
    JMenu submenu = new JMenu("Slivka");

    String[] services = { "clustalo", "clustalw", "probcons", "muscle", "mafft", "tcoffee" };
    String[] names = { "ClustalO", "ClustalW2", "Probcons", "Muscle", "Mafft", "TCoffe" };

    for (int i = 0; i < services.length; i++)
    {
      try
      {
        SlivkaWSInstance instance = new SlivkaWSInstance(client,
            client.getService(services[i]));

        JMenuItem defaultMenuItem = new JMenuItem(
            String.format("%s with defaults", names[i]));
        defaultMenuItem.addActionListener((ActionEvent e) -> {
          AlignmentView msa = alignFrame.gatherSequencesForAlignment();
          if (msa != null)
          {
            new MsaWSClient(instance, alignFrame.getTitle(), msa, false, true,
                alignFrame.getViewport().getAlignment().getDataset(),
                alignFrame);
          }
        });
        submenu.add(defaultMenuItem);

        JMenuItem customMenuItem = new JMenuItem(
            String.format("%s with custom parameters", names[i]));
        customMenuItem.addActionListener((ActionEvent e) -> {
          AlignmentView msa = alignFrame.gatherSequencesForAlignment();
          if (msa != null)
          {
            try
            {
              SlivkaParamSet paramSet = new SlivkaParamSet(
                  instance.getService());
              new MsaWSClient(instance, paramSet, null, true,
                  alignFrame.getTitle(), msa, false, true,
                  alignFrame.getViewport().getAlignment().getDataset(),
                  alignFrame);
            } catch (IOException e1)
            {
              throw new IOError(e1);
            }

          }
        });
        submenu.add(customMenuItem);
      } catch (IOException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    wsmenu.add(submenu);
  }

  @Override
  public void run()
  {

  }
}
